"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var request = require("request");
var zlib = require("zlib");
var HttpClient = /** @class */ (function () {
    function HttpClient(cfg, logger) {
        this.cfg = cfg;
        this.logger = logger;
        if (!cfg) {
            throw new Error("cfg is required");
        }
        if (!logger) {
            throw new Error("logger is required");
        }
    }
    HttpClient.prototype.get = function (urlPath, callback) {
        var _this = this;
        var opts = this.createDefaultOptions(urlPath);
        opts.json = true;
        this.logger.info("About to send GET request. Url:" + opts["url"]);
        request.get(opts, function (err, response, data) {
            if (err) {
                _this.logger.info("Had ERROR in GET request");
                return callback(err, null);
            }
            if (response.statusCode == 404) {
                _this.logger.info("Had 404 in GET request");
                return callback(null, null);
            }
            if (response.statusCode != 200) {
                _this.logger.info("Had != 200 in GET request");
                return callback(new Error("Server returned HTTP Status " + response.statusCode), null);
            }
            _this.logger.info("Fired callback in GET request");
            return callback(null, data);
        });
    };
    HttpClient.prototype.delete = function (urlPath, callback) {
        var opts = this.createDefaultOptions(urlPath);
        opts.json = true;
        request.delete(opts, function (err, response, data) {
            if (err) {
                return callback(err, null);
            }
            if (response.statusCode == 404) {
                return callback(null, null);
            }
            if (response.statusCode != 200) {
                return callback(new Error("Server returned HTTP Status " + response.statusCode), null);
            }
            return callback(null, data);
        });
    };
    HttpClient.prototype.post = function (requestData, urlPath, callback) {
        var _this = this;
        var bufferToSend = new Buffer(JSON.stringify(requestData));
        var shouldZip = this.cfg.compressRequests;
        this.logger.info("Sending buffer:" + bufferToSend.toString());
        if (shouldZip) {
            zlib.gzip(bufferToSend, function (err, compressedBuf) {
                if (err) {
                    //Fallback to sending data uncompressed
                    _this.logger.warn("Failed while trying to compress the request data. Sending uncompressed data instead. Error: ", err);
                    _this.internalPost(urlPath, bufferToSend, null, callback);
                }
                else {
                    _this.internalPost(urlPath, compressedBuf, { "Content-Encoding": "gzip" }, callback);
                }
            });
        }
        else {
            this.internalPost(urlPath, bufferToSend, null, callback); //Submit buffer as-is
        }
    };
    HttpClient.prototype.internalPost = function (urlPath, buffer, additionalHeaders, callback) {
        var _this = this;
        try {
            var opts = this.createDefaultOptions(urlPath);
            opts.body = buffer;
            if (additionalHeaders) {
                for (var header in additionalHeaders) {
                    opts.headers[header] = additionalHeaders[header];
                }
            }
            this.logger.info("About to send POST request. Url:" + opts.url);
            request.post(opts, function (err, response, data) {
                if (err) {
                    _this.logger.error("Failed sending POST request. Error:", err);
                    return callback(err, null);
                }
                if (response.statusCode >= 400) {
                    _this.logger.error("Failed sending POST request. Got 400+ status code. response.statusCode:", response.statusCode);
                    return callback(new Error("HTTP error " + response.statusCode), null);
                }
                _this.logger.info("Sent POST requst. Response: ", response);
                return callback(null, response);
            });
        }
        catch (err) {
            this.logger.error("Failed sending Http POST to :'" + urlPath + "'. Error:" + err);
            callback(err, null);
        }
    };
    HttpClient.prototype.allowUntrustedCertificates = function () {
        //For more information: https://stackoverflow.com/questions/17383351/how-to-capture-http-messages-from-request-node-library-with-fiddler/22996076#22996076
        process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';
    };
    HttpClient.prototype.createDefaultOptions = function (urlPath) {
        //get
        var opts = {
            url: this.cfg.server + urlPath,
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + this.cfg.token
            }
        };
        //TODO: Verify if we need it. Nadav, 21.12.18
        opts["compressed"] = true;
        if (this.cfg.proxy != null) {
            opts.proxy = this.cfg.proxy;
            this.allowUntrustedCertificates();
        }
        return opts;
    };
    return HttpClient;
}());
exports.HttpClient = HttpClient;
